package com.ruoyi.oa.mapper;

import java.util.List;
import com.ruoyi.oa.domain.CmcDeviceApproval;

/**
 * cmc设备审批Mapper接口
 * 
 * @author cmc
 * @date 2024-03-05
 */
public interface CmcDeviceApprovalMapper 
{
    /**
     * 查询cmc设备审批
     * 
     * @param deviceApplyId cmc设备审批主键
     * @return cmc设备审批
     */
    public CmcDeviceApproval selectCmcDeviceApprovalByDeviceApplyId(String deviceApplyId);

    /**
     * 查询cmc设备审批列表
     * 
     * @param cmcDeviceApproval cmc设备审批
     * @return cmc设备审批集合
     */
    public List<CmcDeviceApproval> selectCmcDeviceApprovalList(CmcDeviceApproval cmcDeviceApproval);

    /**
     * 新增cmc设备审批
     * 
     * @param cmcDeviceApproval cmc设备审批
     * @return 结果
     */
    public int insertCmcDeviceApproval(CmcDeviceApproval cmcDeviceApproval);

    /**
     * 修改cmc设备审批
     * 
     * @param cmcDeviceApproval cmc设备审批
     * @return 结果
     */
    public int updateCmcDeviceApproval(CmcDeviceApproval cmcDeviceApproval);

    /**
     * 删除cmc设备审批
     * 
     * @param deviceApplyId cmc设备审批主键
     * @return 结果
     */
    public int deleteCmcDeviceApprovalByDeviceApplyId(String deviceApplyId);

    /**
     * 批量删除cmc设备审批
     * 
     * @param deviceApplyIds 需要删除的数据主键集合
     * @return 结果
     */
    public int deleteCmcDeviceApprovalByDeviceApplyIds(String[] deviceApplyIds);
}
